package cn.com.ty.lift.enterprise.oa.controller;

import cn.com.ty.lift.enterprise.common.CountPage;
import cn.com.ty.lift.enterprise.oa.dto.AttendanceRequest;
import cn.com.ty.lift.enterprise.oa.dto.AttendanceResponse;
import cn.com.ty.lift.enterprise.oa.entity.Attendance;
import cn.com.ty.lift.enterprise.oa.entity.MaintenanceCompany;
import cn.com.ty.lift.enterprise.oa.service.AttendanceService;
import cn.com.ty.lift.enterprise.oa.service.MaintenanceCompanyService;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.time.LocalTime;

/**
 * <p>
 * 打卡签到记录 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("/oa/attendance")
public class AttendanceController {

    private AttendanceService attendanceService;
    private MaintenanceCompanyService maintenanceCompanyService;

    /**
     * @description 根据id查找
     * @date 2019/12/6 9:44
     * @param
     * @return
     */
    @PostMapping("findOne")
    public RestResponse findOne(@RequestBody AttendanceRequest request) {
        Long id = request.getId();
        if(null == id || id <= 0){
            return RestResponse.failParam();
        }
        Attendance entity = attendanceService.getById(id);
        if(null == entity){
            return RestResponse.fail();
        }
        return RestResponse.success(entity);
    }

    /**
     * @description 根据条件查询
     * @date 2019/12/6 9:44
     * @param
     * @return
     */
    @PostMapping("list")
    public RestResponse list(@RequestBody AttendanceRequest request){
        String name = request.getUsername();
        if(StrUtil.isNotBlank(name)){
            request.setUsername(StrUtil.format("%{}%",name));
        }

        IPage<AttendanceResponse> pages = attendanceService.pageByCondition(request);
        if(null == pages){
            return RestResponse.fail();
        }
        long count = attendanceService.count();
        CountPage<AttendanceResponse> countPage = new CountPage<>(count,pages);
        return RestResponse.success(countPage);
    }

    /**
     * @description 设置打卡时间
     * @date 2019/12/20 16:30
     * @param
     * @return
     */
    @PostMapping("setupTime")
    public RestResponse setupTime(@RequestBody AttendanceRequest request){
        LocalTime topTime = request.getTopTime();
        LocalTime downTime = request.getDownTime();
        if ((topTime != null && downTime == null) || (topTime == null && downTime != null)) {
            return RestResponse.fail("请设置上午的上下班时间");
        }
        if (topTime.isAfter(downTime)) {
            return RestResponse.fail("上午的上下班时间有误");
        }
        LocalTime topTimePm = request.getTopTimePm();
        LocalTime downTimePm = request.getDownTimePm();
        if ((topTimePm != null && downTimePm == null) || (topTimePm == null && downTimePm != null)) {
            return RestResponse.fail("请设置下午的上下班时间");
        }
        if (topTimePm.isAfter(downTimePm)) {
            return RestResponse.fail("下午的上下班时间有误");
        }
        LocalTime topTimeOt = request.getTopTimeOt();
        LocalTime downTimeOt = request.getDownTimeOt();
        if ((topTimeOt != null && downTimeOt == null) || (topTimeOt == null && downTimeOt != null)) {
            return RestResponse.fail("请设置加班的上下班时间");
        }
        if (topTimeOt.isAfter(downTimeOt)) {
            return RestResponse.fail("加班的上下班时间有误");
        }
        long mtCompanyId = request.getMtCompanyId();
        MaintenanceCompany entity = maintenanceCompanyService.getById(mtCompanyId);
        if(null == entity){
            return RestResponse.failParam();
        }
        entity.setTopTime(topTime);
        entity.setDownTime(downTime);
        entity.setTopTimePm(topTimePm);
        entity.setDownTimePm(downTimePm);
        entity.setTopTimeOt(topTimeOt);
        entity.setDownTimeOt(downTimeOt);
        boolean result = maintenanceCompanyService.saveOrUpdate(entity);
        if(result){
            return RestResponse.success(result);
        }
        return RestResponse.fail();
    }

    /**
     * 打卡
     * 11 上午上班 12 上午下班
     * 21 下午上班 22 下午下班
     * 31 加班上班 32 加班下班
     *
     * @param request
     * @return
     */
    @PostMapping("clock")
    public RestResponse clock(@RequestBody AttendanceRequest request) {
        String coordinate = request.getCoordinate();
        String address = request.getAddress();
        if (StrUtil.equalsAny(coordinate, address)) {
            return RestResponse.fail("缺少地址信息");
        }
        Long mtCompanyId = request.getMtCompanyId();
        if (null == mtCompanyId || mtCompanyId <= 0) {
            return RestResponse.failParam();
        }
        MaintenanceCompany maintenanceCompany = maintenanceCompanyService.getById(mtCompanyId);
        if (null == maintenanceCompany) {
            return RestResponse.failParam();
        }
        Attendance attendance = new Attendance();
        attendance.setMtCompanyId(mtCompanyId);
        attendance.setUserId(1l);
        attendance.setCoordinate(coordinate);
        attendance.setAddress(address);
        LocalDateTime localDateTime = LocalDateTime.now();
        attendance.setCreateDate(localDateTime);
        LocalTime now = localDateTime.toLocalTime();
        int type = request.getType();
        //状态（是否迟到早退，0：否，1：是）
        switch (type) {
            case 11:
                LocalTime topTime = maintenanceCompany.getTopTime();
                if (null == topTime) {
                    return RestResponse.fail("请先设置上午的上班时间");
                }
                //打卡时间比上班时间早-正常，否则迟到
                attendance.setStatus(now.isBefore(topTime) ? 0 : 1);
                break;
            case 12:
                LocalTime downTime = maintenanceCompany.getDownTime();
                if (null == downTime) {
                    return RestResponse.fail("请先设置上午的下班时间");
                }
                //打卡时间比下班时间晚-正常，否则早退
                attendance.setStatus(now.isAfter(downTime) ? 0 : 1);
                break;
            case 21:
                LocalTime topTimePm = maintenanceCompany.getTopTimePm();
                if (null == topTimePm) {
                    return RestResponse.fail("请先设置下午的上班时间");
                }
                attendance.setStatus(now.isBefore(topTimePm) ? 0 : 1);
                break;
            case 22:
                LocalTime downTimePm = maintenanceCompany.getDownTimePm();
                if (null == downTimePm) {
                    return RestResponse.fail("请先设置下午的下班时间");
                }
                attendance.setStatus(now.isAfter(downTimePm) ? 0 : 1);
                break;
            case 31:
                LocalTime topTimeOt = maintenanceCompany.getTopTimeOt();
                if (null == topTimeOt) {
                    return RestResponse.fail("请先设置加班的上班时间");
                }
                attendance.setStatus(now.isBefore(topTimeOt) ? 0 : 1);
                break;
            case 32:
                LocalTime downTimeOt = maintenanceCompany.getDownTimeOt();
                if (null == downTimeOt) {
                    return RestResponse.fail("请先设置加班的下班时间");
                }
                attendance.setStatus(now.isAfter(downTimeOt) ? 0 : 1);
                break;
            default:
                return RestResponse.failParam();
        }
        return RestResponse.success("打卡成功");
    }

    /**
     * 返回维保公司打卡时间信息
     * @param request
     * @return
     */
    @PostMapping("time")
    public RestResponse time(@RequestBody AttendanceRequest request) {
        Long mtCompanyId = request.getMtCompanyId();
        if (null == mtCompanyId || mtCompanyId <= 0) {
            return RestResponse.failParam();
        }
        MaintenanceCompany maintenanceCompany = maintenanceCompanyService.getById(mtCompanyId);
        if (null == maintenanceCompany) {
            return RestResponse.failParam();
        }
        return RestResponse.success(maintenanceCompany);
    }
}
